'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator;

function _load_regenerator() {
  return _regenerator = _interopRequireDefault(require('babel-runtime/regenerator'));
}

var _bluebird;

function _load_bluebird() {
  return _bluebird = require('bluebird');
}

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path;

function _load_path() {
  return _path = _interopRequireDefault(require('path'));
}

var _dist;

function _load_dist() {
  return _dist = _interopRequireDefault(require('./dist'));
}

var _constants;

function _load_constants() {
  return _constants = require('../../../constants');
}

var _logger;

function _load_logger() {
  return _logger = _interopRequireDefault(require('../../../logger/logger'));
}

var _links;

function _load_links() {
  return _links = require('../../../links');
}

var _utils;

function _load_utils() {
  return _utils = require('../../../utils');
}

var _bitId;

function _load_bitId() {
  return _bitId = require('../../../bit-id');
}

var _path2;

function _load_path2() {
  return _path2 = require('../../../utils/path');
}

var _source;

function _load_source() {
  return _source = _interopRequireDefault(require('../../../scope/models/source'));
}

var _dataToPersist;

function _load_dataToPersist() {
  return _dataToPersist = _interopRequireDefault(require('./data-to-persist'));
}

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * Dist paths are by default saved into the component's root-dir/dist. However, when dist is set in bit.json, the paths
 * are in the consumer-root/dist.target dir. If dist.entry is set, the dist.entry part is stripped from the dists paths.
 * (according to some additional conditions. See shouldDistEntryBeStripped()).
 * If there is originallySharedDir and the component is IMPORTED, it is stripped as well.
 *
 * These modifications of the paths are taken care in different stages depends on the scenario.
 * 1) using 'bit build'.
 * First, the sharedOriginallyDir is stripped (happens in consumer-component.build()). There are two scenarios here:
 *   a) the component wasn't change since the last build. It loads the dists from the model and strip the
 *      sharedOriginallyDir. (see the !needToRebuild case of build()).
 *   b) the component was changed. It re-builds it. The dists path are cloned from the files, since the files are
 *      sharedOriginallyDir stripped (because they loaded from the filesystem), so will be the dists files.
 * Next, the dist.entry is stripped. This is done when the dists are written into the file-system,  (see writeDists()).
 *
 * 2) using 'bit import'.
 * When converting the component from model to consumer-component, the sharedOriginallyDir is stripped. (see
 * stripOriginallySharedDir() ).
 * Then, Before writing the dists to the file-system, the dist-entry is taken care of. (see writeDists() ).
 *
 * 3) using 'bit link'.
 * When linking authored components, we generate an index file from node_modules/component-name to the main dist file.
 * It might happen during the import, when updateDistsPerConsumerBitJson() was running already, and it might happen
 * during the 'bit link' command. Therefore, before linking, the updateDistsPerConsumerBitJson() is running while making
 * sure it doesn't run twice.
 * (see node-modules-linker.linkToMainFile() and calculateMainDistFileForAuthored()).
 *
 * The opposite action is taken when a component is tagged. We load the component from the file-system while the dist
 * paths might be stripped from consumer dist.entry and originallySharedDir.
 * Then, before writing them to the model, we first add the originallySharedDir and then the dist.entry. We make sure
 * there were stripped before adding them. (See this.toDistFilesModel() function and the comment there)
 */
var Dists = function () {
  // populated only after getDistDirForConsumer() is called
  // makes sure to not update twice
  function Dists(dists) {
    _classCallCheck(this, Dists);

    this.writeDistsFiles = true;
    this.areDistsInsideComponentDir = true;
    this.distEntryShouldBeStripped = false;
    this._distsPathsAreUpdated = false;

    this.dists = dists || []; // cover also case of null (when it comes from the model)
  } // changed only when importing a component


  _createClass(Dists, [{
    key: 'isEmpty',
    value: function isEmpty() {
      return !this.dists.length;
    }
  }, {
    key: 'get',
    value: function get() {
      return this.dists;
    }
  }, {
    key: 'getAsReadable',
    value: function getAsReadable() {
      return this.dists.map(function (file) {
        return file.toReadableString();
      });
    }

    /**
     * When dists are written by a consumer (as opposed to isolated-environment for example), the dist-entry and
     * dist-target are taken into account for calculating the path.
     * By default, the dists path is inside the component. If dist attribute is populated in bit.json, the path is
     * relative to consumer root.
     */

  }, {
    key: 'getDistDirForConsumer',
    value: function getDistDirForConsumer(consumer, componentRootDir) {
      var consumerBitJson = consumer.bitJson;
      if (consumer.shouldDistsBeInsideTheComponent()) {
        // should be relative to component
        this.distsRootDir = (_path || _load_path()).default.join(componentRootDir, (_constants || _load_constants()).DEFAULT_DIST_DIRNAME);
      } else {
        // should be relative to consumer root
        if (consumerBitJson.distEntry) componentRootDir = componentRootDir.replace(consumerBitJson.distEntry, '');
        var distTarget = consumerBitJson.distTarget || (_constants || _load_constants()).DEFAULT_DIST_DIRNAME;
        this.areDistsInsideComponentDir = false;
        this.distsRootDir = (_path || _load_path()).default.join(distTarget, componentRootDir);
      }
      return this.distsRootDir;
    }
  }, {
    key: 'getDistDir',
    value: function getDistDir(consumer, componentRootDir) {
      if (consumer) return this.getDistDirForConsumer(consumer, componentRootDir);
      this.distsRootDir = (_path || _load_path()).default.join(componentRootDir, (_constants || _load_constants()).DEFAULT_DIST_DIRNAME);
      return this.distsRootDir;
    }
  }, {
    key: 'updateDistsPerConsumerBitJson',
    value: function updateDistsPerConsumerBitJson(id, consumer, componentMap) {
      var _this = this;

      if (this._distsPathsAreUpdated || this.isEmpty()) return;
      var newDistBase = this.getDistDir(consumer, componentMap.getRootDir());
      var distEntry = consumer ? consumer.bitJson.distEntry : undefined;
      var shouldDistEntryBeStripped = function shouldDistEntryBeStripped() {
        if (!distEntry || componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.NESTED) return false;
        var areAllDistsStartWithDistEntry = function areAllDistsStartWithDistEntry() {
          return _this.dists.map(function (dist) {
            return dist.relative.startsWith(distEntry);
          }).every(function (x) {
            return x;
          });
        };
        if (componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.AUTHORED) {
          return areAllDistsStartWithDistEntry();
        }
        // it's IMPORTED. We first check that rootDir starts with dist.entry, it happens mostly when a user imports into
        // a specific directory (e.g. bit import --path src/). Then, we make sure all dists files start with that
        // dist.entry. In a case when originallySharedDir is the same as dist.entry, this second check returns false.
        // $FlowFixMe
        return componentMap.rootDir.startsWith(distEntry) && areAllDistsStartWithDistEntry();
      };
      var distEntryShouldBeStripped = shouldDistEntryBeStripped();
      if (distEntryShouldBeStripped) {
        // $FlowFixMe
        (_logger || _load_logger()).default.debug(`stripping dist.entry "${distEntry}" from ${id}`);
        this.distEntryShouldBeStripped = true;
      }
      var getNewRelative = function getNewRelative(dist) {
        if (distEntryShouldBeStripped) {
          // $FlowFixMe distEntry is set when distEntryShouldBeStripped
          return dist.relative.replace(distEntry, '');
        }
        return dist.relative;
      };
      this.dists.forEach(function (dist) {
        return dist.updatePaths({ newBase: newDistBase, newRelative: getNewRelative(dist) });
      });
      this._distsPathsAreUpdated = true;
    }
  }, {
    key: 'stripOriginallySharedDir',
    value: function stripOriginallySharedDir(originallySharedDir, pathWithoutSharedDir) {
      this.dists.forEach(function (distFile) {
        var newRelative = pathWithoutSharedDir(distFile.relative, originallySharedDir);
        distFile.updatePaths({ newBase: distFile.base, newRelative });
      });
    }

    /**
     * write dists files to the filesystem
     */

  }, {
    key: 'writeDists',
    value: function () {
      var _ref = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee(component, consumer) {
        var writeLinks = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
        var dataToPersist;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.getDistsToWrite(component, consumer, writeLinks);

              case 2:
                dataToPersist = _context.sent;

                if (dataToPersist) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', null);

              case 5:
                if (consumer) dataToPersist.addBasePath(consumer.getPath());
                _context.next = 8;
                return dataToPersist.persistAllToFS();

              case 8:
                return _context.abrupt('return', this.dists.map(function (distFile) {
                  return distFile.path;
                }));

              case 9:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function writeDists(_x, _x2) {
        return _ref.apply(this, arguments);
      }

      return writeDists;
    }()

    /**
     * In case there is a consumer and dist.entry should be stripped, it will be done before writing the files.
     * The originallySharedDir should be already stripped before accessing this method.
     */

  }, {
    key: 'getDistsToWrite',
    value: function () {
      var _ref2 = (0, (_bluebird || _load_bluebird()).coroutine)( /*#__PURE__*/(_regenerator || _load_regenerator()).default.mark(function _callee2(component, consumer) {
        var writeLinks = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : true;
        var dataToPersist, componentMap, linksInDist;
        return (_regenerator || _load_regenerator()).default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                if (!(this.isEmpty() || !this.writeDistsFiles)) {
                  _context2.next = 2;
                  break;
                }

                return _context2.abrupt('return', null);

              case 2:
                if (!(writeLinks && !consumer)) {
                  _context2.next = 4;
                  break;
                }

                throw new Error('getDistsToWrite expects to get consumer when writeLinks is true');

              case 4:
                dataToPersist = new (_dataToPersist || _load_dataToPersist()).default();
                componentMap = void 0;

                if (consumer) {
                  componentMap = consumer.bitMap.getComponent(component.id, { ignoreVersion: true });
                  this.updateDistsPerConsumerBitJson(component.id, consumer, componentMap);
                }
                dataToPersist.addManyFiles(this.dists);

                if (!(writeLinks && componentMap && componentMap.origin === (_constants || _load_constants()).COMPONENT_ORIGINS.IMPORTED)) {
                  _context2.next = 13;
                  break;
                }

                _context2.next = 11;
                return (0, (_links || _load_links()).getLinksInDistToWrite)(component, componentMap, consumer);

              case 11:
                linksInDist = _context2.sent;

                dataToPersist.merge(linksInDist);

              case 13:
                return _context2.abrupt('return', dataToPersist);

              case 14:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function getDistsToWrite(_x4, _x5) {
        return _ref2.apply(this, arguments);
      }

      return getDistsToWrite;
    }()

    // In case there are dist files, we want to point the index to the main dist file, not to source.
    // This important since when you require a module without specify file, it will give you the file specified under this key
    // (or index.js if key not exists)

  }, {
    key: 'calculateMainDistFile',
    value: function calculateMainDistFile(componentMainFile) {
      if (this.writeDistsFiles && this.areDistsInsideComponentDir) {
        // Take the only dist file if there is only one or search for one with the same name as the main source file
        var mainFile = this.dists && this.dists.length === 1 ? this.dists[0].relative : (0, (_utils || _load_utils()).searchFilesIgnoreExt)(this.dists, componentMainFile, 'relative', 'relative');
        if (mainFile) return (_path || _load_path()).default.join((_constants || _load_constants()).DEFAULT_DIST_DIRNAME, mainFile);
      }
      return componentMainFile;
    }

    /**
     * authored components have the dists outside the components dir and they don't have rootDir.
     * it returns the main file or main dist file relative to consumer-root.
     */

  }, {
    key: 'calculateMainDistFileForAuthored',
    value: function calculateMainDistFileForAuthored(componentMainFile, consumer) {
      if (this.isEmpty()) return componentMainFile;
      var getMainFileToSearch = function getMainFileToSearch() {
        if (!consumer.bitJson.distEntry) return componentMainFile;
        var distEntryNormalized = (_path || _load_path()).default.normalize(consumer.bitJson.distEntry);
        return componentMainFile.replace(`${distEntryNormalized}${(_path || _load_path()).default.sep}`, '');
      };
      var mainFileToSearch = getMainFileToSearch();
      var distMainFile = (0, (_utils || _load_utils()).searchFilesIgnoreExt)(this.dists, mainFileToSearch, 'relative', 'relative');
      if (!distMainFile) return componentMainFile;
      var distTarget = consumer.bitJson.distTarget || (_constants || _load_constants()).DEFAULT_DIST_DIRNAME;
      return (_path || _load_path()).default.join(distTarget, distMainFile);
    }
  }, {
    key: 'toDistFilesModel',
    value: function toDistFilesModel(consumer, originallySharedDir, compiler) {
      var _this2 = this;

      // when a component is written to the filesystem, the originallySharedDir may be stripped, if it was, the
      // originallySharedDir is written in bit.map, and then set in consumerComponent.originallySharedDir when loaded.
      // similarly, when the dists are written to the filesystem, the dist.entry may be stripped, if it was, the
      // consumerComponent.dists.distEntryShouldBeStripped is set to true.
      // because the model always has the paths of the original author, in case part of the path was stripped, add it
      // back before saving to the model. this way, when the author updates the components, the paths will be correct.
      var addSharedDirAndDistEntry = function addSharedDirAndDistEntry(pathStr) {
        var withSharedDir = originallySharedDir ? (_path || _load_path()).default.join(originallySharedDir, pathStr) : pathStr;
        var withDistEntry = _this2.distEntryShouldBeStripped // $FlowFixMe
        ? (_path || _load_path()).default.join(consumer.bitJson.distEntry, withSharedDir) : withSharedDir;
        return (0, (_path2 || _load_path2()).pathNormalizeToLinux)(withDistEntry);
      };

      if (this.isEmpty() || !compiler) return null;

      return this.get().map(function (dist) {
        return {
          name: dist.basename,
          relativePath: addSharedDirAndDistEntry(dist.relative), // $FlowFixMe
          file: (_source || _load_source()).default.from(dist.contents),
          test: dist.test
        };
      });
    }

    /**
     * the formula is distTarget + (customModuleDir - distEntry).
     * e.g. distTarget = 'dist', customDir = 'src', distEntry = 'src'.
     * node_path will be 'dist' + 'src' - 'src' = 'dist'.
     * another example, distTarget = 'dist', customDir = 'src/custom', distEntry = 'src'. result: "dist/custom"
     */

  }], [{
    key: 'getNodePathDir',
    value: function getNodePathDir(consumer) {
      var resolveModules = consumer.bitJson.resolveModules;
      if (!resolveModules || !resolveModules.modulesDirectories || !resolveModules.modulesDirectories.length) return null;
      var distTarget = consumer.bitJson.distTarget || (_constants || _load_constants()).DEFAULT_DIST_DIRNAME;
      var distEntry = consumer.bitJson.distEntry;
      var nodePaths = resolveModules.modulesDirectories.map(function (moduleDir) {
        var isRelative = function isRelative(str) {
          return str.startsWith('./') || str.startsWith('../');
        };
        if (!distEntry) return (_path || _load_path()).default.join(distTarget, moduleDir);
        var distEntryRelativeToModuleDir = (0, (_utils || _load_utils()).pathRelativeLinux)(distEntry, moduleDir);
        if (isRelative(distEntryRelativeToModuleDir)) {
          // moduleDir is outside the distEntry, ignore the distEntry
          return (_path || _load_path()).default.join(distTarget, moduleDir);
        }
        return (_path || _load_path()).default.join(distTarget, distEntryRelativeToModuleDir);
      });
      return nodePaths.map(function (nodePath) {
        return consumer.toAbsolutePath(nodePath);
      }).map((_path2 || _load_path2()).pathNormalizeToLinux).join((_constants || _load_constants()).NODE_PATH_SEPARATOR);
    }
  }]);

  return Dists;
}();

exports.default = Dists;